/* ************************************************************************** */
/** Descriptive File Name

   @ Author
    Richard Wall
 
  @ Date
    Created:    May 28, 2016
    Revised:    March 29, 2017, added CLEAR and DIVIDE
                May 16, 2017, modified DIVIDE to use switch2Binary
    Verified:   May 16, 2017 - all functions

  @Company
    Digilent       

   @File Name
    calc.h
 
  @Summary
    Simple calculator key definitions and operations.
 
  @Description
    Defines switches SW0 through SW3 as nibble X and SW4 through SW7 as nibble
    Y, Defines BTND, BTNC, BTNR, BTNU, and BTNL as ADD, SUB, MUlT, DIV, and CLEAR.
 */
/* ************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include "calc.h"
#include "hardware.h"
#include "swDelay.h"
// #include "led7.h"
#include <plib.h>

/* ************************************************************************** */
/* ************************************************************************** */
// Section: Local Functions                                                   */
/* ************************************************************************** */
/* ************************************************************************** */

// *****************************************************************************
/**
    @Function
        int get_xy(int *x, int *y); 
    @Summary
        Simple calculator emulator.
    @Description
        * Translates the SWITCH inputs SW0 through SW3 as value x and SW4 
        * through SW7 as value y.
        * Data enter to the calculator uses hexadecimal format
    @Precondition
        * LED IO pins have been initialized as outputs
    @Parameters
        @param param1: pointer to x in range of 0b0000 to 0b1111 ( 0 - 15d )
        @param param2: pointer to y in range of 0b0000 to 0b1111 ( 0 - 15d )
    @Returns
        * 1   Indicates an error occurred
        * 0   Indicates an error did not occur
    @Remarks
        * Returned error flag indicates the value of either x or y is out of
        * range 0 through 15.
    @Example
        @code
             error_flag =  get_xy(int *x, int *y); 
*/
int get_xy(int *x, int *y)
{

int error_flag = 0;
unsigned int sw;
    
    sw = Switch2Binary();    // Read binary encoded switches
 
// Parse into x and y values
    *x = sw & 0x0f;  //(SW0() | SW1()<<1 | SW2()<<2 | SW3()<<3);
    *y = (sw & 0xf0) >> 4;  //(SW4() | SW5()<<1 | SW6()<<2 | SW7()<<3);
    
// Test for range errors
    if( *x > 15 )
    {
        *x = 0;
        error_flag |= 1;
    }
    if( *y > 15 )
    {
        *y = 0;
        error_flag |= 1;
    }
    
    return error_flag;
}

/* *****************************************************************************
 End of File calc.h
 */
