/* ************************************************************************** */
/** Descriptive File Name

  @ Author
    Richard Wall
 
  @ Date
    Created:    May 28, 2016
    Revision:   January 23, 2017 - added decimal point control
    Verified:   May 16, 2017

  @Company
    Digilent Inc.

  @File Name
    LED7.h

  @ Date
    Created:    May 28, 2016
    Revision:   January 23, 2017 - added decimal point control
    Verified:   May 16, 2017


  @Summary
    Four Digit - Seven segment display constants.

  @Description
    Defines the IO pins and decimal digit output for a seven segment display.
 */
/* ************************************************************************** */

#ifndef _LED7_H_    /* Guard against multiple inclusion */
#define _LED7_H_

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include "hardware.h"
#include <plib.h>

    /* ************************************************************************** */
    /* ************************************************************************** */
    /* Section: Constants                                                         */
    /* ************************************************************************** */
    /* ************************************************************************** */

    /*  This file defines the port and pin assignments for the 7 segment LED 
     * displays.  Macros are used extensively to implement abstraction of the
     * bit-bang control required because the segment LEDs are not assigned to
     * contiguous pins on a single port.  
     */


    /* ************************************************************************** */
    /** Descriptive Constant Name

      @Summary
     Seven segment display definition and control macros.
      
      @Description
       LED segment control definitions require individual bit control. "SET_xx"
       operators declare the segment pins as outputs. "SEG_xx" and "DIG_xx" 
       operators assign values either "0" or "1" to the outputs.  "SEG_xx" 
       operators control the LED segments and "DIG_ANx" select the display 
       digit. Digits are selected by asserting the digit anode low.
        
       Reference: https://en.wikipedia.org/wiki/Seven-segment_display  
     */
/* Macros that set the PIC32 pins as digital outputs for LED segments */
#define SET_CA()    TRISGCLR = BIT_12
#define SET_CB()    TRISACLR = BIT_14
#define SET_CC()    TRISDCLR = BIT_6
#define SET_CD()    TRISGCLR = BIT_13
#define SET_CE()    TRISGCLR = BIT_15
#define SET_CF()    TRISDCLR = BIT_7
#define SET_CG()    TRISDCLR = BIT_13
#define SET_DP()    TRISGCLR = BIT_14

/* Controls the ON and OFF state of LED segments. LED segments are active low.*/
#define SEG_CA(b);  {if(b) LATGCLR = BIT_12; else LATGSET = BIT_12;}  //RG12
#define SEG_CB(b);  {if(b) LATACLR = BIT_14; else LATASET = BIT_14;}  //RA14
#define SEG_CC(b);  {if(b) LATDCLR = BIT_6;  else LATDSET = BIT_6; }  //RD5
#define SEG_CD(b);  {if(b) LATGCLR = BIT_13; else LATGSET = BIT_13;}  //RG13
#define SEG_CE(b);  {if(b) LATGCLR = BIT_15; else LATGSET = BIT_15;}  //RG15
#define SEG_CF(b);  {if(b) LATDCLR = BIT_7 ; else LATDSET = BIT_7; }  //RD6
#define SEG_CG(b);  {if(b) LATDCLR = BIT_13; else LATDSET = BIT_13;}  //RD13
#define SEG_DP(b);  {if(b) LATGCLR = BIT_14; else LATGSET = BIT_14;}  //RG14

/* Macros that set the PIC32 pins as digital outputs for LED Digit display */
#define SET_AN0()   TRISBCLR = BIT_12   // RB12
#define SET_AN1()   TRISBCLR = BIT_13   // RB13
#define SET_AN2()   PORTSetPinsDigitalOut(IOPORT_A, BIT_9);    // RA9
#define SET_AN3()   PORTSetPinsDigitalOut(IOPORT_A, BIT_10);   // RA10

/* Controls the ON and OFF state of digit displays. These controls are one hot
 * code - active low. 
 */
#define DIG_AN0(b); {if(b) LATBSET = BIT_12; else LATBCLR = BIT_12;}  //RB12
#define DIG_AN1(b); {if(b) LATBSET = BIT_13; else LATBCLR = BIT_13;}  //RB13
#define DIG_AN2(b); {if(b) LATASET = BIT_9;  else LATACLR = BIT_9; }  //RA9
#define DIG_AN3(b); {if(b) LATASET = BIT_10; else LATACLR = BIT_10;}  //RA10

/* The following macros turn individual segment LEDS on or off according to
  the value to be displayed by pulling the LED cathodes low.
*/
#define SEG_LED0 {SEG_CA(1); SEG_CB(1);SEG_CC(1);SEG_CD(1);SEG_CE(1);SEG_CF(1);SEG_CG(0);SEG_DP(0);}
#define SEG_LED1 {SEG_CA(0); SEG_CB(1);SEG_CC(1);SEG_CD(0);SEG_CE(0);SEG_CF(0);SEG_CG(0);SEG_DP(0);}
#define SEG_LED2 {SEG_CA(1); SEG_CB(1);SEG_CC(0);SEG_CD(1);SEG_CE(1);SEG_CF(0);SEG_CG(1);SEG_DP(0);}
#define SEG_LED3 {SEG_CA(1); SEG_CB(1);SEG_CC(1);SEG_CD(1);SEG_CE(0);SEG_CF(0);SEG_CG(1);SEG_DP(0);}
#define SEG_LED4 {SEG_CA(0); SEG_CB(1);SEG_CC(1);SEG_CD(0);SEG_CE(0);SEG_CF(1);SEG_CG(1);SEG_DP(0);}
#define SEG_LED5 {SEG_CA(1); SEG_CB(0);SEG_CC(1);SEG_CD(1);SEG_CE(0);SEG_CF(1);SEG_CG(1);SEG_DP(0);}
#define SEG_LED6 {SEG_CA(1); SEG_CB(0);SEG_CC(1);SEG_CD(1);SEG_CE(1);SEG_CF(1);SEG_CG(1);SEG_DP(0);}
#define SEG_LED7 {SEG_CA(1); SEG_CB(1);SEG_CC(1);SEG_CD(0);SEG_CE(0);SEG_CF(0);SEG_CG(0);SEG_DP(0);}
#define SEG_LED8 {SEG_CA(1); SEG_CB(1);SEG_CC(1);SEG_CD(1);SEG_CE(1);SEG_CF(1);SEG_CG(1);SEG_DP(0);}
#define SEG_LED9 {SEG_CA(1); SEG_CB(1);SEG_CC(1);SEG_CD(0);SEG_CE(0);SEG_CF(1);SEG_CG(1);SEG_DP(0);}
#define SEG_NEG  {SEG_CA(0); SEG_CB(0);SEG_CC(0);SEG_CD(0);SEG_CE(0);SEG_CF(0);SEG_CG(1);SEG_DP(0);}
#define SEG_OFF  {SEG_CA(0); SEG_CB(0);SEG_CC(0);SEG_CD(0);SEG_CE(0);SEG_CF(0);SEG_CG(0);SEG_DP(0);}
#define DIGITS_OFF {DIG_AN0(1); DIG_AN1(1); DIG_AN2(1); DIG_AN3(1);}
// *****************************************************************************
// *****************************************************************************
// Section: Interface Functions
// *****************************************************************************
// *****************************************************************************
void seg7_init(void);   // initializes the 4 digit- 7 segment LED display 
void clr_dsp(void);     // Turns off all LED segments (blanks display) 
void set_digit(int dsp, int value, int dp); // Sets Segment LEDS and activates digit 
void dsp_digit(int value, int dp); // Sets segment LEDs for specific value
void led_number(int value); // Displays a value (0 - 9999) 
void test_7seg_leds(void);  // Seven segment display test sequence 

#endif /* _LED7_H_ */

/* *****************************************************************************
 End of File led7.h
 */
