/* ************************************************************************** */
/** Descriptive File Name main for Lab 2b - Stepper motor control
  @ Author
    Richard Wall
 
  @ Date
    Created:    June 3, 2016
    Revisions:  October 11, 2016 - Updated MPLAB
                January 24, 2107 - New stepper motor
    Verified:   May 16 12, 2017
 * 
  @Company
    Digilent Inc.

  @File Name
    main.c

  @Development Environment
    MPLAB X IDE x3.61 - http://www.microchip.com/mplab/mplab-x-ide 
	XC32 1.43 - http://www.microchip.com/mplab/compilers
	PLIB 3/7/20162 - http://www.microchip.com/SWLibraryWeb/product.aspx?product=PIC32%20Peripheral%20Library

  @Summary
    Stepper motor example program.

  @Description
    Implements stepper motor direction, mode, and speed control.
    Uses Timer 2 as the time base and functions in a foreground / background 
    manner. Stepper motor operating parameters are polled during the foreground
    execution. Output are generated in the execution of background code. 
   
   This project was last tested using MPLAB X Rev. 3.61 and XC32 Rev. 1.43
 
  @Remarks
    Uses XC32 peripheral library. See 
 http://ww1.microchip.com/downloads/en/DeviceDoc/32bitPeripheralLibraryGuide.pdf
 http://www.microchip.com/SWLibraryWeb/product.aspx?product=PIC32%20Peripheral%20Library
 
    This project specifically targets the Digilent motor available from
    http://store.digilentinc.com/stepper-motor/. For motor details see
    https://reference.digilentinc.com/_media/stepper_motor:290-028-1_ts-24byj48a-25-pxw.pdf
 
 ************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include "config_bits.h"    // PIC32MX370 bit configuration
#include "hardware.h"       // Defines hardware IO
#include <plib.h>           // Peripherial library 
#include <stdlib.h>

#include "swDelay.h"        // Software millisecond time delay
#include "led7.h"           // 4 digit - 7 segment display control

#include "stepper.h"
/* ************************************************************************** */
/* ************************************************************************** */
/* Section: File Scope or Global Data                                         */
/* ************************************************************************** */
/* ************************************************************************** */

/*  These global variables are needed to communicate with the Timer 1 ISR. */
int step_delay = 25;        // Default initial value
int step_count = 1000;      // Default initial value
int step_dir = CCW;         // Default initial value
int step_mode = HALF_STEP;  // Default initial value
extern int led_flag;        // Enable seven segment display
extern int led_value;      // Value to be displayed on the LED display

/* Local Function declaration */
static void project_setup(void);    // Initializes application

/** 
  @Function
    int main( void ); 

  @Summary
    Initializes the hardware platform and polls switches and push buttons to 
    determine the stepper motor operational parameters.

  @Description
    This routine is contains an initialization portion and an infinite loop
    portion.  The switches and push buttons are read and interpreted in the
    foreground code. All outputs are generated in the background execution of 
    Timer 2 ISR execution.
  
  @Precondition
    None

  @Parameters
    None

  @Returns
    The system has failed if the last statement is executed. 
	  EXIT_FAILURE (1)   Indicates an error occurred
      EXIT_SUCCESS (0)   Indicates an error did not occur

  @Remarks
    The returned value is never used but supplied to suppress compiler error and warnings.
 
 */

int main( void )
{
int rpm;
    project_setup();
    led_flag = 1;
    while(1)
    {
        if(BNTD())              // Determine step mode
        {
            step_mode = FULL_STEP;
        }
        else
        {
            step_mode = HALF_STEP;            
        }
        
        if(BNTR())              // Determine rotation direction
        {
            step_dir = CW;
        }
        else
        {
            step_dir = CCW;            
        }
        

        if(!step_count)         // Step count measures 10 revolutions
        {
            step_count= 10*STEPSPREREV; // Set BP here to stop motor
        }

        rpm = Switch2Binary();  // Poll switches to determine set speed
        if(rpm == 0)            // Avoid divide by zero
        {
            step_delay = RPM2MSPERHALFSTEP;
            led_value = 0;
        }
        else
        {
            led_value = (rpm * 100) / 8;
            step_delay = (int) ((RPM2MSPERHALFSTEP * step_mode * 8) / ( rpm ));
        }
    } 
    return (EXIT_FAILURE);      // Should never reach this statement
}

/* ************************************************************************** */
/* ************************************************************************** */
// Section: Local Functions                                                   */
/* ************************************************************************** */
/* ************************************************************************** */

/*******************************************************************************
  Function:
    static void project_setup(void)

  Summary:
    Sets up processor platform for application.

  Description:
    This routine configures the IO pins used in this application and starts a
    Timer 2 interrupt for once a millisecond.

  Precondition:
    None

  Parameters:
    None

  Returns:
    None

  Example:
    <code>
    project_setup();
    </code>

  Remarks:
    None
  *****************************************************************************/
static void project_setup(void)
{
#define T2_TICK  GetPeripheralClock()/10000  // Timer 2 ticks per / 0.1 ms

/* Call library initializations */    
    Hardware_Setup();       // Initialize push buttons and LEDs
    seg7_init();            // Initialize IO pins for 7 segment display
    stepper_init();         // Initialize stepper motor pins 

/* Timing instrumentation */
    TRISEbits.TRISE0 = 0;   // Use LCD DB0 for Timer 2 ISR timing output
    LATEbits.LATE0 = 0;

/* Configure Timer2 to roll over once each 1/10 millisecond */
    OpenTimer2(T2_ON | T2_SOURCE_INT | T2_PS_1_1, T2_TICK - 1);
/* set up the timer interrupt with a priority of 2 */
    ConfigIntTimer2(T2_INT_ON | T2_INT_PRIOR_2);
}

/*******************************************************************************
  Function:
    void __ISR(_TIMER_2_VECTOR, IPL2SOFT) Timer2Handler(void)

  Summary:
    Services a Timer 2 interrupt.

  Description:
    This interrupt service routine executes the required processing each Timer 2
    interrupt.  Two tasks are executed in this ISR: updates the seven-segment 
    LED display and generates a single stepper motor step. The seven-segment LED
    display display each of the four digits for one millisecond in a round-
    robin fashion. The stepper motor step is taken at the end of the step delay 
    period. Testing shows that this ISR takes between 2us and 4.25us.

  Precondition:
    IO has been initialized and global variables have been set.

  Parameters:
    None - ISR cannot have variables passed to it.

  Returns:
    None - ISR cannot return a value.

  Example:
    None. ISRs cannot be called directly. They can only be initiated after the 
    interrupt flag has been set either in software or hardware.
 
  Remarks:
    This routine uses and/or modifies the following global variables:
    step_delay, step_mode, and step_dir.
    DB0 (RE0) is used to time the execution time of the ISR   
    The execution time of ISRs should be kept to a minimum to minimize latency of lower priority interrupts and foreground tasks.
  *****************************************************************************/
void __ISR(_TIMER_2_VECTOR, IPL2SOFT) Timer2Handler(void)
{
static int delay = 0;               // 1/10 ms delay counter for stepper motor 
static int led_disp = 3;            // Seven-segment LED digit display index
static int led_digit;               // Seven-segment LED digit display value
static int ms_cntr = 10;            // 1 ms counter for 7 segment LED 

    LATEbits.LATE0 = 1;             // ISR performance timing uses DB0

// Test for time to update stepper motor position
    if(--delay <= 0)                
    {
        delay = step_delay;         // Reset the delay counter 
        step(step_dir, step_mode);  // Move stepper motor 
        invLED0();                  // toggle step LED0 each step
    }

    INTClearFlag( INT_T2 );   // Clear the interrupt flag
    LATEbits.LATE0 = 0;       // ISR performance timing uses LCD DB0
} /* End of Timer2Handler ISR */

/* ************************  End of File main.c ************************** */
