/* ************************************************************************** */
/** Descriptive File Name: stepper.c is a demonstration of an implementation of a 4 wire stepper motor.

  @Company
    Digilent   
 
  @Author
    Dr. Richard W Wall
 
  @Date
    June 2, 2016
 
  @File Name
    stepper.h

  @Summary
    4 wire and 5/6 wire stepper motor control.

  @Description
    This file provides the definitions and interface control of a stepper motor 
    in full or half steps.
 */
/* ************************************************************************** */

#ifndef _STEPPER_MOTOR_H    /* Guard against multiple inclusion */
#define _STEPPER_MOTOR_H


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

/* This section lists the other files that are included in this file.
 */

#include "hardware.h"
#include <plib.h>
/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Constants                                                         */
/* ************************************************************************** */
/* ************************************************************************** */

/* ************************************************************************** */
/** Descriptive Constant Name
*/
//#define A1IN1bit        BIT_3       //RB3
//#define A2IN1bit        BIT_8       //RE8
//#define B1IN1bit        BIT_9       //RE9
//#define B2IN1bit        BIT_5       //RB5

#define STEPPER_MODE_DIR 	PORTSetDigitalOut(IOPORT_F, BIT_1) //RF1  
#define STEPPER_MODE(a)    	PORTSetBits(IOPORT_F, BIT_1)       //RF1  

#define FULL_STEP       (int) 2
#define HALF_STEP       (int) 1
#define CW              (int) 1
#define CCW             (int) -1

#define STEPSPREREV         1625    //48	//steps per revolution
#define RPM2MSPERHALFSTEP   370/2     // 6250    //1/10 ms per half step per RPM  

    // *****************************************************************************
    // *****************************************************************************
    // Section: Data Types
    // *****************************************************************************
    // *****************************************************************************

    /*  Provide interface with the stepper motor library functions
     */

    // *****************************************************************************
    // *****************************************************************************
    // Section: Interface Functions
    // *****************************************************************************
    // *****************************************************************************

    /*    4 wire and 5/6 wire stepper motor control. */

    // *****************************************************************************
    /**
/** 
  @Function
    void steppep_init(void); 

  @Summary
    Initializes the PIC32 pins for output

  @Description
    The four stepper motor phase and mode pins are set as outputs and set to
    zero (off). 

  @Precondition
    None
    
  @Parameters
    @param param1 
        None
  @Returns
    None
  @Remarks
    All analog capable IO pins must have the analog functionality disabled
  @Example
    @code
    steppep_init(void); 
 */
void stepper_init(void);

/** 
  @Function
    void stepper(int direction, int mode);
 
   @Summary
    Implements a single stepper motor step based on direction and mode.

  @Description
    Determines the stepper motor control code based upon the
    present state, direction, and mode. This rendition of the
    stepper motor FSM uses an algorithm to determine the next
    state that is representative of a ROM based FSM.
 
  @Precondition
    Processor pin must have been set for digital output. See stepper_init();
    
  @Parameters
    @param param1 
        Direction of rotation --> +1 or -1
    @param param2
        Step mode --> HALF STEP --> +1 or FULL STEP --> 2
 
  @Returns
    None
 
  @Remarks
    Stepper outputs must be bit banged and hence incorrect output codes may  
    be generated for some transitions.

  @Example
    @code
    stepper(CW, HALF_STEP); 
 */
void step(int dir, int mode);


#endif /* _EXAMPLE_FILE_NAME_H */

/* *****************************************************************************
 End of File
 */
