/* ************************************************************************** */
/** Descriptive File Name: Lab 2c is a demonstration of an implementation of a
    finite state machine to the control of the speed, direction and step mode
    of a 4 wire stepper motor.

  @ Author
    Richard Wall
 
  @ Date
    Created:    October 8, 2016
    Verified:   May 20,2017
 
   @Company
    Digilent Inc.

  @File Name
    main.c

  @Development Environment
    MPLAB X IDE x3.61 - http://www.microchip.com/mplab/mplab-x-ide 
	XC32 1.43 - http://www.microchip.com/mplab/compilers
	PLIB 3/7/20162 - http://www.microchip.com/SWLibraryWeb/product.aspx?product=PIC32%20Peripheral%20Library

  @Summary
    momentary contact push button operation as Push-on / push-off switch.

  @Description
    This project implements a non-blocking push-on / push-off push button 
    operation including switch contact bouncing elimination.

  @Remarks
    Uses XC32 peripheral library. See 
 http://ww1.microchip.com/downloads/en/DeviceDoc/32bitPeripheralLibraryGuide.pdf
 http://www.microchip.com/SWLibraryWeb/product.aspx?product=PIC32%20Peripheral%20Library

  See notes in Section 12.1.4 of the PIC32MX370 users manual
    http://ww1.microchip.com/downloads/en/DeviceDoc/60001185D.pdf
 
 ****   This application uses a peripheral bus clock set to 10MHz.  ****
    PBCLK set for 10MHz, Set parameters in config_bits.h and hardware.h
 
************************************************************************** */

/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */

// System included files
#include "config_bits.h"
#include "hardware.h"
#include <plib.h>

// Application included files
#include "swDelay.h"
#include "main.h"

/* ************************************************************************** */
/* ****************************************************************************
 * The purpose of this example code is to demonstrate the use of 
 * peripheral library macros and functions supporting the PIC32MX general 
 * purpose Type A Timer 1. The application is monitoring three momentary 
 * contact push buttons that are normally low the implement three independently
 * operated push-on / push-off switches.
 *
 * Platform:     Basys MX3 
 *
 * Features demonstrated:
 *      - switch-case state machine
 *      - Software delay using the core timer
  *
 * Description:
 *
 * Oscillator Configuration Bit Settings: config_bits.h
 * Basys MX3 hardware initialization is provided by Hardware_Setup.c 
 * Notes:
 *        - Peripheral Clock: PBCLK = 80E6 / 8 = 10E6 
 
 ****************************************************************************/

// System Constant Definitions

#define BTN_DLY     30  // Debounce delay time - ms
#define NUM_BTN     3   // Number of push buttons
#define PB0         1   // BTNR
#define PB1         2   // BTNC
#define PB2         4   // BTND
#define CN_OFF      0   // Change notification disabled
#define CN_ON       1   // Change notification enabled

// Global variable for ISR communications
unsigned int pbTrig = 0;  // push button change notice (ISR))
unsigned int pbPos = 0;   // Condition (pressed/released) when triggered (ISR)
unsigned int pb_Dly[NUM_BTN] = {0};     // Button trigger delay (ISR)
unsigned int sw_output[NUM_BTN] = {0};  // push-on/off switch output (Global)

// Local function prototypes
void push_on_off( void );
static void os_setup(void);
static void cn(int button, int flag);

/* ******************************************************************** */
int main(void)
{
    Hardware_Setup();           // Hardware initialization
    os_setup();                 // Operating system initialization
    
    TRISEbits.TRISE0 = 0;       // Timing output
    LATEbits.LATE0 = 0;
    
    while(1)
    {
        if(pbTrig) 
        {
            LATEbits.LATE0 = 1;        
            push_on_off();      // Call button toggle only when button changes 
            LATEbits.LATE0 = 0;        
        }
    }
    return (EXIT_FAILURE);  // System has failed is this line is reached
}

/* START FUNCTION DESCRIPTION ********************************************
 * push_on_off
 * SYNTAX:         void push_on_off( void );
 * KEYWORDS:       User interface operating system
 * DESCRIPTION:    This function polls a global variable that indicates which
 *                 one or ones of three momentary contact push buttons has been
 *                 pressed. Two variables describe the switch: the trigger and 
 *                 the position. The position indicates the wheather the button
 *                 was in the pressed or released condition when the trigger was
 *                 pressed.  
 *                 
 * PARAMETERS:     None
 * RETURN:         None
 * NOTES:          CN interrupts run at a group priority level 1 and Timer1 runs
 *                 runs at a group priority level 2.  Since the push buttons are
 *                 connected to a disparate range of ports and pins, the CN
 *                 interrupt initialization is lengthy.
 * 
 *                 Execution time - 2.4959 us
 * 
 * END DESCRIPTION *******************************************************/
void push_on_off( void )
{
static int sw_state[NUM_BTN] = {0};
int button = 0;

    for(button=0; button < NUM_BTN; button++)   // Check all push buttons
    {
        if( pbTrig & (1 << button ))            // Test button trigger flag
        {
            if(!pb_Dly[button])                 // Test for in delay period
            {
                switch( sw_state[button] )
                {
                    case 0:     // Button pressed - TURN switch on
                        if((pbPos & (1 << button)) && !pb_Dly[button])  
                        {                       // on-exit actions
                            switch(button)
                            {
                                case 0:
                                    setLED0(1);
                                    break;
                                case 1:
                                    setLED1(1);
                                    break;
                                case 2:
                                    setLED2(1);
                                    break;
                            }   
                            sw_output[button] = 1;
                            pb_Dly[button] = BTN_DLY;
                            sw_state[button] = 1;   // Advance state
                        }
                        break;
                    case 1:             // Button released - switch stays on
                        if(!(pbPos & (1 << button)) && !pb_Dly[button])   
                        {               // on-exit actions
                            pb_Dly[button] = BTN_DLY;
                            pbTrig &= ~(1<<button); // Reset button trigger flag
                            sw_state[button] = 2;   // Advance state
                        }
                        break;
                    case 2:             // Button pressed - turn switch off
                        if((pbPos & (1 << button)) && !pb_Dly[button])    
                        {               // on-exit actions      
                            switch(button)
                            {
                                case 0:
                                    setLED0(0);
                                    break;
                                case 1:
                                    setLED1(0);
                                    break;
                                case 2:
                                    setLED2(0);
                                    break;
                            }
                            sw_output[button] = 0;
                            pb_Dly[button] = BTN_DLY;
                            sw_state[button] = 3;   // Advance state
                        }
                        break;
                    case 3:             // Button released - switch stays off
                        if(!(pbPos & (1 << button)) && !pb_Dly[button])   
                        {       // on-exit actions  
                            pb_Dly[button] = BTN_DLY;       
                            pbTrig &= ~(1<<button); // Reset button trigger flag
                            sw_state[button] = 0;   // Advance back to state 0
                        }
                        break;
                }
                pbTrig &= ~(1 << button );  // Reset trigger
            }
        }
    }
}

/* START FUNCTION DESCRIPTION ********************************************
 * cn
 * SYNTAX:         static void cn(int button, int flag);
 * KEYWORDS:       Change notification enable / disable
 * DESCRIPTION:    Turns off or on associated CN interrupts 
 * PARAMETER 1:    button - button identifier
 * PARAMETER 1:    flag: _OFF or _ON
 * RETURN:         None
 * NOTES:          29 cycles - 0.3625 us, Measured - 0.33947 us
 * 
 * END DESCRIPTION *******************************************************/
static void cn(int button, int flag)
{
    switch(button)
    {
        case PB0:
            CNCONBbits.ON = flag;      // Port B change notification
            break;
        case PB1:
            CNCONFbits.ON = flag;      // Port F change notification
            break;
        case PB2:
            CNCONFbits.ON = flag;      // Port F change notification
            break;
    }
}

/* START FUNCTION DESCRIPTION ********************************************
 * CNHandler
 * SYNTAX:         void __ISR(_CHANGE_NOTICE_VECTOR, IPL1SOFT) CNHandler(void);
 * KEYWORDS:       Change notification interrupt
 * DESCRIPTION:    Turns off associated CN interrupts and sets both the push 
 *                 button trigger and state flags.
 * PARAMETERS:     None
 * RETURN:         None
 * NOTES:          The pbPos indicates whether the push button (active high)
 *                 is pressed or not. pbTrig indicates that the push button
 *                 has been pressed or released. SInce the push buttons being 
 *                 monitored are on different ports, three individual CN 
 *                 interrupt flags must be polled and possibly reset.
 * 
 *                 Measured ISR time - 1.2456 us
 * 
 * END DESCRIPTION *******************************************************/
void __ISR(_CHANGE_NOTICE_VECTOR, IPL1SOFT) CNHandler(void) 
{	
    if(IFS1bits.CNBIF)              // BTNR - RB8
    {
        if(CNSTATB & BTNR_bit)      // Check if CN on BTNR - RB8
        {
            cn(PB0,CN_OFF);           // Disable Port B change notification
            pbTrig |= PB0;          // Set switch trigger flag
            if(BNTR())
            {
                pbPos |= PB0;     // Set switch state flag on
            }
            else
            {
                pbPos &= ~PB0;    // Set switch state flag off
            }
            IFS1bits.CNBIF = 0;     // Clear Port B CN interrupt flag
        }
    }
    if(IFS1bits.CNFIF)              // Check if CN on BTNC - RF0
    {
        if(CNSTATF & BTNC_bit)      // Check if CN on RF0 
        {
            cn(PB1,CN_OFF);           // Disable Port F change notification
            pbTrig |= PB1;          // Set switch trigger flag
            if(BNTC())
            {
                pbPos |= PB1;     // Set switch state flag on
            }
            else
            {
                pbPos &= ~PB1;    // Set switch state flag off
            }
            IFS1bits.CNFIF = 0;     // Clear Port F CN interrupt flag
        }
    }

    if( IFS1bits.CNAIF )            // Check if CN on BTND - RA15
    {
        if(CNSTATA & BTND_bit)      // Check  if CN on RA15
        {
            cn(PB2,CN_OFF);           // Disable Port A change notification
            pbTrig |= PB2;          // Set switch trigger flag
            if(BNTD())
            {
                pbPos |= PB2;     // Set switch state flag on
            }
            else
            {
                pbPos &= ~PB2;    // Set switch state flag off
            }
            IFS1bits.CNAIF = 0;     // Clear Port A CN interrupt flag
        }
    }
}

/* START FUNCTION DESCRIPTION ********************************************
 * Timer1Handler
 * SYNTAX:         void __ISR(_TIMER_1_VECTOR, IPL2SOFT) Timer1Handler(void);
 * KEYWORDS:       millisecond, hardware delay, Timer A, timer
 * DESCRIPTION:    Decrements multiple push button debounce delays once each ms 
 *                 until zero.
 * PARAMETERS:     None
 * RETURN:         None
 * 
 * NOTES:          Measured ISR execution time - 2.658 us
 * 
 * END DESCRIPTION *******************************************************/
void __ISR(_TIMER_1_VECTOR, IPL2SOFT) Timer1Handler(void)
{
int btn;
    for(btn = 0; btn < NUM_BTN; btn++)
    {
        if(pb_Dly[btn])             // Decrement delay if not zero
        {
            pb_Dly[btn]--;    
        }
        else
        {
            cn((1 << btn), CN_ON);    // Enable CN - delay is over
        }
    }
    INTClearFlag( INT_T1 );         // Clear the interrupt flag
}

/* START FUNCTION DESCRIPTION ********************************************
 * os_setup
 * SYNTAX:         static void os_setup(void);
 * KEYWORDS:       User interface operating system
 * DESCRIPTION:    Sets up change notification interrupts for three momentary
 *                 contact push buttons, a one ms timer interrupt, and enables
 *                 multi nested interrupt environment.
 * PARAMETERS:     None
 * RETURN:         None
 * NOTES:          CN interrupts run at a group priority level 1 and Timer1 runs
 *                 runs at a group priority level 2.  Since the push buttons are
 *                 connected to a disparate range of ports and pins, the CN
 *                 interrupt initialization is lengthy.
 * END DESCRIPTION *******************************************************/
static void os_setup(void)
{
#define MS_TICK     GetPeripheralClock()/1000    

// The push buttons on the Basys MX3 board active high and have pull down 
// resistors already installed so no internal pull up or pull down resistors
// should be enabled. 
    
// Configure BTNR for change notification  - pb[0]
    ANSELBbits.ANSB8 = 0;   // Disable analog input
    TRISBbits.TRISB8 = 1;   // Set for digital input

    CNPDBbits.CNPDB8 = 0;   // Disable bit 8 pull down resistor
    CNPUBbits.CNPUB8 = 0;   // Disable bit 8 pull up resistor
    CNENBbits.CNIEB8 = 1;   // Enable bit 8 change notification on 
    CNCONBbits.ON = 1;      // Enable Port B change notification
    IEC1bits.CNBIE = 1;     // Enable Bort B CN interrupts 

// Configure BTNC for change notification - PB[1]
    TRISFbits.TRISF0 = 1;   // Set for digital input

    CNPDFbits.CNPDF0 = 0;   // Disable bit 0 pull down resistor
    CNPUFbits.CNPUF0 = 0;   // Disable bit 0 pull up resistor
    CNENFbits.CNIEF0 = 1;   // Enable bit 0 change notification on 
    CNCONFbits.ON = 1;      // Enable Port F change notification
    IEC1bits.CNFIE = 1;     // Enable Bort F CN interrupts 
    
// Configure BTND for change notification - PB[2]
    TRISAbits.TRISA15 = 1;  // Set for digital input

    CNPDAbits.CNPDA15 = 0;  // Disable bit 15 pull down resistor
    CNPUAbits.CNPUA15 = 0;  // Disable bit 15 pull up resistor
    CNENAbits.CNIEA15 = 1;  // Enable bit 15 change notification on 
    CNCONAbits.ON = 1;      // Enable Port A change notification   
    IEC1bits.CNAIE = 1;     // Enable Bort A CN interrupts 
    
// Configure BTNC for change notification
    TRISFbits.TRISF0 = 1;   // Set for digital input

    CNPDFbits.CNPDF0 = 0;   // Disable bit 0 pull down resistor
    CNPUFbits.CNPUF0 = 0;   // Disable bit 0 pull up resistor
    CNENFbits.CNIEF0 = 1;   // Enable bit 0 change notification on 
    CNCONFbits.ON = 1;      // Enable Port 0 change notification
    
    IEC1bits.CNFIE = 1;     // Enable port F CN interrupts 

// Set CN interrupt level
    IPC8bits.CNIP = 1;      // Set interrupt group level
    IPC8bits.CNIS = 0;      // Set interrupt sub group level
    
// Configure Timer1 to roll over once each second 
    OpenTimer1(T1_ON | T1_SOURCE_INT | T1_PS_1_1, MS_TICK-1);
// set up the timer interrupt with a priority of 2 
    ConfigIntTimer1(T1_INT_ON | T1_INT_PRIOR_2);

// Enable global nested interrupts    
    INTConfigureSystem(INT_SYSTEM_CONFIG_MULT_VECTOR); // Execute only once
    INTEnableInterrupts();  // Execute as needed
} // End of os_setup

/* End of file main.c  */
