/* ************************************************************************** */
/** Descriptive File Name - Running the accelerometer through I2C
  @ Author
    Richard Wall
 
  @ Date
    Created:    June 21, 2016
    Verified:   May 17, 2017

  @Company
    Digilent Inc.      

  @File Name
    Lab4d.c

  @Development Environment
    MPLAB X IDE x3.61 - http://www.microchip.com/mplab/mplab-x-ide 
	XC32 1.43 - http://www.microchip.com/mplab/compilers
	PLIB 3/7/20162 - http://www.microchip.com/SWLibraryWeb/product.aspx?product=PIC32%20Peripheral%20Library

  @Summary
    Demonstration of MMA8652FC accelerometer interface using I2C 

  @Description
    The #define statements and macro C code provide high level access to the 
    Basys MX3 Trainer boards switches, push buttons, and LEDs as well as analog
    inputs.

 @Remarks
    Uses XC32 peripheral library. See 
 http://ww1.microchip.com/downloads/en/DeviceDoc/32bitPeripheralLibraryGuide.pdf
 http://www.microchip.com/SWLibraryWeb/product.aspx?product=PIC32%20Peripheral%20Library
  
 ****   This application uses a peripheral bus clock set to 10MHz.  ****
    PBCLK set for 10 MHz, Set parameters in config_bits.h and hardware.h

     
 */
/* ************************************************************************** */

/* Included files for platform */
#include "hardware.h"
#include "config_bits.h"
#include "swDelay.h"

/* Included files for system */
/* This included file provides access to the peripheral library functions and
   must be installed after the XC32 compiler. See
http://ww1.microchip.com/downloads/en/DeviceDoc/32bitPeripheralLibraryGuide.pdf
http://www.microchip.com/SWLibraryWeb/product.aspx?product=PIC32%20Peripheral%20Library
 */
#include <plib.h>
#include <stdio.h>

/* Included files for application */
#include "Lab4d.h"
#include "LCDlib.h"             // Basys character LCD
#include "comm_lib.h"           // Basys MX3 UART4
#include "MMA8652FC.h"          // Accelerometer

// *****************************************************************************
/**
    @Function
        int main(void);

    @Summary
        Application code entry point containing execution of one-time setup and
        initialization followed by the run-time infinite loop.

    @Description
        After initialization of all resources, the accelerometer is read once
        each second and the results displayed on the Basys MX3 character display
        and sent to the UART port at 38.4KB 

    @Parameters
        None

    @Returns
        int - exit success is always failure because the execution of the 
        infinite loop should never stop for an embedded system.

***************************************************************************** */

int main(void)
{
int loop_counter = 0;   // Counts number of cycles in while(1) loop 
int result_flag;        // I2C transaction results
int x, y, z;            // Accelerometer data
int status;             // Accelerometer status
char txt_str[40];       // LCD text string

    Hardware_Setup();               // Initialize common Basys MX3 IO pins
    initLCD();                      // Initialize LCD
    uart4_init(38000, NO_PARITY);   // Set UART BAUD for 38400 - no parity
    result_flag = I2C_init(I2C1, 100000);  // Set I2C bit rate for 100000
    if(result_flag)                 // Set LED0 on if I2C correctly initialized
    {    setLED0(ON); }
    else
    {    setLED0(OFF); }              // Set LED0 off if I2C fails to initialized
    mma8562fc_init();               // Initialize accelerometer

    while(1)                        // Application forever loop
    {
        loop_counter++;             // Increment progress counter
        result_flag = mma8562fc_xyz(&status, &x, &y, &z); // Read accelerometer
    
        sprintf(txt_str,"x=%4d y=%4d", x, y);   // Format text for LCD
        clrLCD();                       
        putsLCD(txt_str);           // DIsplay first line on LCD
        gotoLCD(16);
        sprintf(txt_str,"z=%4d",z);
        putsLCD(txt_str);           // DIsplay second line on LCD
        
        printf("%4d status=%d  x=%5d y=%5d z=%5d\n\r",  // Display results on
                loop_counter, status, x, y, z);       // terminal.
        msDelay(LOOP_DELAY);              // Wait 1 second
    }
}

// End of Lab4d.c
