/* ************************************************************************** */
/** Descriptive File Name - Driver for writing to flash memory through SPI

  @Company
    Digilient
 
  @Author
    Richard Wall

  @Date
    July 8, 2016
    Rev. August 8, 2016

  @File Name
    flash_mem.h

  @Summary
    Software driver for reading and writing to flash memory using SPI interface.
      @Summary        .

  @Description
    Hardware handler for the S25FL132 flash memory device.
    See data sheet posted http://www.cypress.com/file/196886/download

  @Remarks
    Uses XC32 peripheral library. See 
 http://ww1.microchip.com/downloads/en/DeviceDoc/32bitPeripheralLibraryGuide.pdf
 http://www.microchip.com/SWLibraryWeb/product.aspx?product=PIC32%20Peripheral%20Library
  
 ****   This application uses a peripheral bus clock set to 10MHz.  ****
    PBCLK set for 10 MHz, Set parameters in config_bits.h and hardware.h

/* ************************************************************************** */

#ifndef _FLASH_MEMORY_H    /* Guard against multiple inclusion */
#define _FLASH_MEMORY_H

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

// Platform common included files
#include "hardware.h"
#include "SPI.h"

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Constants                                                         */
/* ************************************************************************** */
/* ************************************************************************** */


/* ************************************************************************** */
    /** Descriptive Constant Name
  @Summary
        Constants for controlling the access to and protection of FLASH memory
 
  @Description
        Commands vary in length from a single byte to several bytes. Each 
        command begins with an instruction code and may be followed by address 
        bytes, a mode byte, read latency (dummy / don't care) cycles, or data
        bytes. Commands are completed with the rising edge of edge CS#. Clock 
        relative sequence diagrams for each command are included in the command
        descriptions. All read commands can be completed after any data bit.
        However, all command that Write, Program or Erase must complete on a
        byte boundary (CS# driven high after a full 8 bits have been clocked)
        otherwise the command will be ignored. This feature further protects
        the device from inadvertent writes. Additionally, while the memory is
        being programmed or erased, all commands except for Read Status
        Register and Suspend commands will be ignored until the program or
        erase cycle has completed. When the Status Register is being written,
        all commands except for Read Status Register will be ignored until the
        Status Register write operation has completed.
     
**************************************************************************** */

// Define the FLASH chip select pin
#define SPIFLASH_SEL                    LATFbits.LATF8      // RF8 - OK
#define SPIFLASH_CS_TRIS                TRISFbits.TRISF8    // OK
    
#define SPIFLASH_PAGE_SIZE              256
#define SPIFLASH_SECTOR_SIZE            4096
#define SPIFLASH_SECTOR_COUNT           1024
#define SPIFLASH_ADDR_MAX               0x40000
#define SPIFLASH_BLOCK                  0x3FF00

// Command Set (Reset Commands)
#define SPIFLASH_CMD_SW_RESET_EN        0x66
#define SPIFLASH_CMD_SW_RESET           0x99
#define SPIFLASH_CMD_CONT_READ_RESET    0xFF

// Command Set (ID, Security Commands)
// Command Set (Configuration, Status, Erase, Program Commands (1))
#define SPIFLASH_CMD_READ               0x03    // SPI Flash opcode: Read up up to 25MHz
#define SPIFLASH_CMD_READ_FAST			0x0B    // SPI Flash opcode: Read up to 50MHz with 1 dummy byte
#define SPIFLASH_CMD_ERASE_4K			0x20    // SPI Flash opcode: 4KByte sector erase
#define SPIFLASH_CMD_ERASE_32K			0x52    // SPI Flash opcode: 32KByte block erase
#define SPIFLASH_CMD_ERASE_SECTOR		0xD8    // SPI Flash opcode: 64KByte block erase
#define SPIFLASH_CMD_ERASE_ALL			0x60    // SPI Flash opcode: Entire chip erase
#define SPIFLASH_CMD_WRITE				0x02    // SPI Flash opcode: Write one byte (or a page of up to 256 bytes, depending on device)
#define SPIFLASH_CMD_WRITE_WORD_STREAM	0xAD    // SPI Flash opcode: Write continuous stream of 16-bit words (AAI mode); available on SST25VF016B (but not on SST25VF010A)
#define SPIFLASH_CMD_WRITE_BYTE_STREAM	0xAF    // SPI Flash opcode: Write continuous stream of bytes (AAI mode); available on SST25VF010A (but not on SST25VF016B)
#define SPIFLASH_CMD_RDSR_1				0x05    // SPI Flash opcode: Read Status Register 1
#define SPIFLASH_CMD_RDSR_2				0x35    // SPI Flash opcode: Read Status Register 2
#define SPIFLASH_CMD_EWSR				0x50    // SPI Flash opcode: Enable Write Status Register
#define SPIFLASH_CMD_WRSR				0x01    // SPI Flash opcode: Write Status Register
#define SPIFLASH_CMD_WREN				0x06    // SPI Flash opcode: Write Enable
#define SPIFLASH_CMD_WRDI				0x04    // SPI Flash opcode: Write Disable / End AAI mode
#define SPIFLASH_CMD_RDID				0x90    // SPI Flash opcode: Read ID
#define SPIFLASH_CMD_JEDEC_ID			0x9F    // SPI Flash opcode: Read JEDEC ID
#define SPIFLASH_CMD_EBSY				0x70    // SPI Flash opcode: Enable write BUSY status on SO pin
#define SPIFLASH_CMD_DBSY				0x80    // SPI Flash opcode: Disable write BUSY status on SO pin
#define SPIFLASH_CMD_PWRDWN_DEVID       0xAB    // SPI Flash opcode: Release from Deep-Power-Down / Device ID
#define SPIFLASH_CMD_DEEP_PWR_DOWN      0xB9    // SPI Flash opcode: Set for Deep-Power-Down

// These next two bits are the only volitle SR1 status bits.
#define SPIFLASH_BUSY_STATUS_BIT        0x01    // SPI Flash busy bit in SR1 status register
#define SPIFLASH_WEL_STATUS_BIT         0x02    // SPI Flash Write Enable Latch bit in SR1 status register

// *****************************************************************************
// *****************************************************************************
// Section: Interface Functions
// *****************************************************************************
// *****************************************************************************

/*                  SPI SERIAL FLASH LIBRARY            */
// *****************************************************************************
/**
    @Function
            int flash_mem_init(void) 

    @Summary
         Initializes SPI port and FLASH memory for READ and WRITE operations

    @Description
        Initate the flash memory device and corresponding SPI port

    @Precondition
        None.

    @Parameters
        None
    
      @Returns
        The SPI bit rate

      @Remarks
        None
 
      @Example
        @code
        {
            unsigned int bitrate;
            bitrate = flash_mem_init()
        }
**/
    unsigned int flash_mem_init(void);

/**
    @Function
        void SPIFLASH_ReleasePowerDownGetDeviceID(BYTE *rd); 

    @Summary
        Released the SPI FLASH device from deep power down mode and reads the
        FLASH ID.

    @Description
        Wake up the SPI flash device and read the Flash ID


    @Precondition
        Both SPI and Flash memory must have been initialized

    @Parameter 1
        Pointer to the BYTE array to contain the data to write
 
    @Returns
        None

    @Remarks
        Refer to page 65 of the data sheet. 
        See http://www.cypress.com/file/196886/download
 
    @Example
        @code
        {
            BYTE spi_pkt[10] = {0};
            SPIFLASH_ReleasePowerDownGetDeviceID(spi_pkt); 
        }
**/
    void SPIFLASH_ReleasePowerDownGetDeviceID(BYTE *rd);
    

/**
    @Function
        void SPIFLASH_CmdSet(BYTE cmd, BYTE *rd, int len) 

    @Summary
        Writes a single byte command to the SPI FLASH and reads the N bytes of
        status response

    @Description
        SPI FLASH commands and settings are used to setup operations or 
        determine present operating status. The number of bytes expected in
        response to the command must be supplied as determined by the data
        sheet.


    @Precondition
        Both SPI and Flash memory must have been initialized

    @Parameter 1
        Command byte
    @Parameter 2
        BYTE pointer to the response buffer array(1 to 6 bytes)
    @Parameter 3
        Number of BYTEs of response to read
 
      @Returns
        None

      @Remarks
        Refer to page 65 of the data sheet. 
        See http://www.cypress.com/file/196886/download
 
      @Example
        @code
        {
            BYTE spi_pkt[10] = {0};
            SPIFLASH_CmdSet(SPIFLASH_CMD_PWRDWN_DEVID, spi_pkt, 5);
        }
**/
    void SPIFLASH_CmdSet(BYTE cmd, BYTE *rd, int len);

    
/**
    @Function
        void SPIFLASH_WritePage(int nBytes, unsigned char *pbWrData, 
                           unsigned char data_addr)

    @Summary
        Writes and Reads a specified number of bytes to a page or portion of a 
        page of Flash memory. This is a blocking function.

    @Description
      Memory can only be programmed that has been previously erased (0xFF). A 
      maximum of 256 bytes can be written and can never across a page boundary.
      (Meaning advancing LSB address byte from 0xff to 0x00.) 
 
      This function performs the following operations:
        1. Checks to for number of bytes to be written less than 256 and will
           not attempt to write across a page boundary (xxFF to xx00).
        2. Reads the FLASH page or portion of FLASH page into temporary buffer
        3. Checks to see that portion of FLASH memory has been erased
        4. Returns an error of 1 if not erased otherwise continues
        5. Sets WEL bit with SPIFLASH_CMD_WREN command.
        6. Polls the SR1 status register until the WEL bits is read as 1 and
           the BUSY bit is read as zero.
        7. Sets the write header buffer for SPIFLASH_CMD_WRITE command followed
           by the 24 bit starting address.
        8. Sets CS low.
        8. Writes the four bytes of the header buffer followed by 1 to 256 
           bytes of data subject to the constraints described above.
        10. Sets CS high
        11. Clears WEL bit with SPIFLASH_CMD_WRDI command.

        Steps 8 through 10 are a single SPI transaction.
    @Precondition
        Both SPI and Flash memory must have been initialized

    @Parameter 1
        Number of BYTEs to write
    @Parameter 2
        Pointer to the BYTE array to contain the data to write
    @Parameter 3
        3 byte (24 bit) starting address
 
    @Returns
        Error flag: 0 -> no error
                    1 -> attempt to write beyond page boundary
    @Remarks
        No write will be attempted is the entire range of memory is not erased
        and an error flag is returned to the caller.
        For additional information see the FLASH data sheet at
            http://www.cypress.com/file/196886/download
 
      @Example
        @code
        {
            int error;
            int len = 100;
            unsigned int addr = 0x1000;
            BYTE data[len] = {0};

            error = SPIFLASH_WritePage(len, data, addr);
        }
**/
    int SPIFLASH_WritePage(int nBytes, unsigned char *pbWrData, 
                           unsigned int data_addr);

/**
    @Function
        void SPIFLASH_WriteData(int nBytes, unsigned char *pbWrData, 
                                unsigned char data_addr)

    @Summary
        This file writes an arbitrary number of bytes to the SPI FLASH memory
        starting at an arbitrary address. There is no limitation to the number
        of bytes read during a single transaction.


    @Description
        Memory can only be programmed that has been previously erased (0xFF). 
        This function parses the data block to conform to the page constraints
        used by the SPIFLASH_WritePage function.
    @Precondition
        Both SPI and Flash memory must have been initialized

    @Parameter 1
        Number of BYTEs to write
    @Parameter 2
        Pointer to the BYTE array to contain the data to write
    @Parameter 3
        3 byte (24 bit) starting address
 
    @Returns
        Error flag: 0 -> no error
                    1 -> attempt to write beyond page boundary
    @Remarks
        For additional information see the FLASH data sheet at
            http://www.cypress.com/file/196886/download
 
      @Example
        @code
        {
            int error;
            int len = 100;
            unsigned int addr = 0x1000;
            BYTE data[len] = {0};

            error = SPIFLASH_WriteData(len, data, addr);
        }
**/
    int SPIFLASH_WriteData(int nBytes, BYTE *pbWrData, unsigned int flashAddr);

/**
    @Function
        void SPIFLASH_ReadData(int nBytes, unsigned char *pbRdData, 
                               unsigned char data_addr);

    @Summary
        Reads specified number of bytes from the flash memory starting at
        the specified address

    @Description
        This file reads an arbitrary number of bytes from the SPI FLASH memory
        starting at an arbitrary address. There is no limitation to the number
        of bytes read during a single transaction.


    @Precondition
        Both SPI and Flash memory must have been initialized

    @Parameter 1
        Number of BYTEs to read
    @Parameter 2
        Pointer to the BYTE array to contain the data read
    @Parameter 3
        Starting address
 
    @Returns
        None

    @Remarks

        For additional information see the FLASH data sheet at
            http://www.cypress.com/file/196886/download
 
      @Example
        @code
        {
            int len = 100;
            unsigned int addr = 0x1000;
            BYTE data[len];

            SPIFLASH_ReadData(len, data, addr);
        }
**/
    void SPIFLASH_ReadData(int nBytes, unsigned char *pbRdData, 
                           unsigned int data_addr);

/**
    @Function
        void SPIFLASH_TrasferBytes(int nBytes, unsigned char *pbRdData, 
                                   unsigned char *pbWrData); 

    @Summary
        Writes and Reads a specified number of bytes.   

    @Description
        The write header (command and starting address) must be contained at 
        the beginning of the pbWrData array. This transfer is limited to the
        minimum of the size of pbWrData or pbRdData which ever is less.


    @Precondition
        Both SPI and Flash memory must have been initialized

    @Parameter 1
        Number of BYTEs to read
    @Parameter 2
        Pointer to the BYTE array to contain the data read
    @Parameter 3
        Address of first byte to be read
 
    @Returns
        None

    @Remarks
        None
 
    @Example
        @code
        {
            int len = 5;
            rdBuf[len];
            wrBuf[len];
            SPIFLASH_TrasferBytes(len, rdBuf, wrBuf);
        }
**/
    void SPIFLASH_TrasferBytes(int nBytes, unsigned char *pbRdData, 
                           unsigned char *pbWrData);
    
/**
    @Function
        void SPIFLASHErase(BYTE com, unsigned int pageAddr); 

    @Summary
        Writes all the bytes in a sector to 0xFF.   

    @Description
        Erases all bytes within a sector of SPI FLASH

    @Precondition
        Both SPI and Flash memory must have been initialized

    @Parameter 1
        Erase command - 
            SPIFLASH_CMD_ERASE_4K   // SPI Flash opcode: 4KByte sector erase
            SPIFLASH_CMD_ERASE_32K	// SPI Flash opcode: 32KByte block erase
            SPIFLASH_CMD_ERASE_SECTOR // SPI Flash opcode: 64KByte block erase
            SPIFLASH_CMD_ERASE_ALL	// SPI Flash opcode: Entire chip erase

    @Parameter 2
        Base address of sector
 
    @Returns
        None

    @Remarks
        This is a self contained erase of all 4096 bytes in a sector.
 
    @Example
        @code
        {
            unsigned int Addr = 0x1000;
            BYTE cmd = SPIFLASH_CMD_ERASE_4K;
            SPIFLASH4KErase(cmd, pageAddr);
        }
**/
    void SPIFLASHErase(BYTE cmd, unsigned int Addr);

#endif /* _FLASH_MEMORY_H */

/* *****************************************************************************
 End of File
 */
