/* ************************************************************************** */
/** Descriptive File Name - Demonstrate the use of flash memory through SPI
  @ Author
    Richard Wall
 
  @ Date
    Created:    July 8, 2016
	Revised: 	October 31, 2016
    Verified:   May 17, 2017

  @Company
    Digilent Inc.      

  @File Name
    Lab4c.c

  @Development Environment
    MPLAB X IDE x3.61 - http://www.microchip.com/mplab/mplab-x-ide 
	XC32 1.43 - http://www.microchip.com/mplab/compilers
	PLIB 3/7/20162 - http://www.microchip.com/SWLibraryWeb/product.aspx?product=PIC32%20Peripheral%20Library

  @Summary
    MIC32MX370 Flash Memory IC interface using SPI 

  @Description
    The #define statements and macro C code provide high level access to the 
    Basys MX3 Trainer boards switches, push buttons, and LEDs as well as analog
    inputs.
    
 **************************************************************************** */

/* Included files for platform */
#include "hardware.h"
#include "config_bits.h"
#include "swDelay.h"

/* Included files for system */
/* This included file provides access to the peripheral library functions and
   must be installed after the XC32 compiler. See
http://ww1.microchip.com/downloads/en/DeviceDoc/32bitPeripheralLibraryGuide.pdf
http://www.microchip.com/SWLibraryWeb/product.aspx?product=PIC32%20Peripheral%20Library
 */
#include <plib.h>
#include <stdio.h>

/* Included files for application */
#include "SPI.h"
#include "LCDlib.h"
#include "comm_lib.h"
#include "flash_mem.h"

// *****************************************************************************
/**
    @Function
        int main(void);

    @Summary
        Application code entry point containing execution of one-time setup and
        initialization followed by the run-time infinite loop.

    @Description
        After initialization of all hardware resources, the flash memory device 
        is read once each second and the results displayed on the Basys MX3 
        character display and sent to the UART port at 38.4KB 

    @Parameters
        None

    @Returns
        int - exit success is always failure because the execution of the 
        infinite loop should never stop for an embedded system.

***************************************************************************** */
int main(void)
{
static BYTE spi_pkt[10] = {0};
static BYTE wrBuffer[1024] = {0};
static BYTE rdBuffer[1024] = {0};

unsigned int nBytes, flashAddr;
unsigned int error = 0;

int i;

    Hardware_Setup();				// Initialize common Basys MX3 IO pins
    initLCD();                      // Initialize LCD
    uart4_init(38400, NO_PARITY);   // Set UART BAUD for 38400 - no parity
    flash_mem_init();				// Initialize flash memory communications

// Initialize test array   
    nBytes = 550;                   // Number of bytes to read
    flashAddr = 100;                // Starting FLASH memory address
    for(i=0; i<nBytes; i++)         // Initialize array for FLASH write
    {
        wrBuffer[i] = (BYTE) ('A' + i);
    }

    SPIFLASH_ReleasePowerDownGetDeviceID(spi_pkt);   // Tested

//  SPIFLASH_CmdSet(SPIFLASH_CMD_PWRDWN_DEVID, spi_pkt, 5);     // Tested
//  SPIFLASH_CmdSet(SPIFLASH_CMD_RDID, spi_pkt, 6);             // Tested
//  SPIFLASH_CmdSet(SPIFLASH_CMD_JEDEC_ID, spi_pkt,4);          // Tested

// ----------------------  Erase sector 0 - Works -----------------------------
//    SPIFLASH_ReadData(nBytes, buffer, flashAddr );            // Tested
//    SPIFLASHErase(SPIFLASH_CMD_ERASE_4K, 0);                  // Tested
//    SPIFLASH_ReadData(nBytes, buffer, flashAddr );            // Tested 
// ----------------------------------------------------------------------------
//    error = SPIFLASH_WritePage(16, buffer, 32 );              // Tested
 
    while(1)
    {
        printf("Press BTNR to write to Flash Memory\n\r");
        if(BNTR())
        {   
            SPIFLASHErase(SPIFLASH_CMD_ERASE_4K, 0);                   // Tested
//          SPIFLASHErase(SPIFLASH_CMD_ERASE_4K, 0x100);               // Tested
            SPIFLASH_ReadData(sizeof(rdBuffer), rdBuffer, flashAddr ); // Tested 
            error = SPIFLASH_WriteData(nBytes, wrBuffer, flashAddr);   // Tested
            printf("Flash written with %d errors. Release BTNR to check.\n\r", 
                    error);
            do
			{
                msDelay(50);	// Delay 50 ms if BTNR is pressed for debounce
            }while(BNTR());
        }
    
        for(i=0; i<sizeof(wrBuffer); i++)     // clear buffer to zero
        {
            rdBuffer[i] = 0;
        }

        SPIFLASH_ReadData(nBytes, rdBuffer, flashAddr);	// Read Flash 
        i = 0;
        error = 0;
        while(!error && (i < nBytes))   // Check data read
        {
            if((rdBuffer[i] != wrBuffer[i]))
            {
              error = 1;  
            }
            else
                i++;
        }
        
        if(error)
        {
            printf("Error at address %d: %d written - %d read\n\r", i+flashAddr, 
                    wrBuffer[i], rdBuffer[i] );
        }
        else
        {
            printf("No memory error!\n\r");
        }
            
        msDelay(1000);                      // Delay 1 second
    }
    return EXIT_FAILURE;
}

// End of Lab4c.c
