/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Digilent   
 
  @Author
    Richard W Wall
 
  @Date
    Created: June 3, 2016
    Verified: May 20, 2017
 
  @File Name
    stepper.c

  @Development Environment
    MPLAB X IDE x3.61 - http://www.microchip.com/mplab/mplab-x-ide 
	XC32 1.43 - http://www.microchip.com/mplab/compilers
	PLIB 3/7/20162 - http://www.microchip.com/SWLibraryWeb/product.aspx?product=PIC32%20Peripheral%20Library

  @Summary
    4 wire and 5/6 wire stepper motor control.

  @Description
    This file provides the control of a stepper motor in full or half steps.

  @ Remarks
    This implementation bit-bangs the IO.  This implementation of stepper is NOT
    the same as the implementation of the stepper motor used in Lab 2b. 
 */
/* ************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include "stepper.h"
#include "hardware.h"
#include <plib.h>

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: File Scope or Global Data                                         */
/* ************************************************************************** */
/* ************************************************************************** */

/* This file is for use with any 4W or 5/6 wire stepper motor provided 
 * the phasing is correct.
 * 
 * Digilent P/N 290-028 - http://store.digilentinc.com/stepper-motor/ 
 *  Phase    Wire color
 *  B1IN       Blue
 *  B2IN       Yellow
 *  A2IN       Orange
 *  A1IN       Pink
 *             Red - No connection
 */ 

/*  These global variables are needed to communicate with the Timer 2 ISR. */
int step_delay = 25;        // Default initial value
int step_count = 1000;      // Default initial value
int step_dir = SM_CCW;      // Default initial value
int step_mode = SM_HALF_STEP;  // Default initial value
int led_flag = 1;           // Enable seven segment display
int led_value;              // Value to be displayed on the LED display

/* ************************************************************************** */
/* ************************************************************************** */
// Section: Public Functions                                                   */
/* ************************************************************************** */
/* ************************************************************************** */

/** 
  @Function
    void steppep_init(void); 

  @Summary
    Initializes the PIC32 pins for output

  @Description
    The four stepper motor phase and mode pins are set as outputs and set to
    zero (off). 

  @Precondition
    None
    
  @Parameters
    @param param1 
        None
  @Returns
    None
  @Remarks
    All analog capable IO pins must have the analog functionality disabled
  @Example
    @code
    steppep_init(void); 
    @code
 */
void stepper_init(void) 
{
    LATFbits.LATF1  =  0;           // Set MODE to zero (Stepper motor)
    LATBbits.LATB3  =  0;           // Turn power off to all phases
    LATEbits.LATE8  =  0;
    LATEbits.LATE9  =  0;
    LATBbits.LATB5  =  0;           

    TRISBbits.TRISB3 = 0;           //RB3
    ANSELBbits.ANSB3 = 0;           // Disable analog input
    TRISEbits.TRISE8 = 0;           //RE3
    TRISEbits.TRISE9 = 0;           //RE9
    TRISBbits.TRISB5 = 0;           //RB5
    ANSELBbits.ANSB5 = 0;           // Disable analog input
    TRISFbits.TRISF1 = 0;           //RF1  
    
    /* Configure Timer2 to roll over once each millisecond */
    OpenTimer2(T2_ON | T2_SOURCE_INT | T2_PS_1_1, T2_TICK-1);
/* set up the timer interrupt with a priority of 3 */
    ConfigIntTimer2(T2_INT_ON | T2_INT_PRIOR_3);
    
/* Enable multi-vector interrupts */
    INTConfigureSystem(INT_SYSTEM_CONFIG_MULT_VECTOR);  /* Do only once */
    INTEnableInterrupts();   /*Do as needed for global interrupt control */

}

/* ************************************************************************** */
/* ************************************************************************** */
// Section: Interface Functions                                               */
/* ************************************************************************** */
/* ************************************************************************** */

/*  Stepper Motor Control
 */

// *****************************************************************************

/** 
  @Function
    void stepper(int direction, int mode);
 
   @Summary
    Implements a single stepper motor step based on direction and mode.

  @Description
    Determines the stepper motor control code based upon the
    present state, direction, and mode. This rendition of the
    stepper motor FSM uses an algorithm to determine the next
    state that is representative of a ROM based FSM.
 
  @Precondition
    Processor pin must have been set for digital output. See stepper_init();
    
  @Parameters
    @param param1 
        Direction of rotation --> +1 or -1
    @param param2
        Step mode --> HALF STEP --> +1 or FULL STEP --> 2
 
  @Returns
    None
 
  @Remarks
    Stepper outputs must be bit banged and hence incorrect output codes may  
    be generated for some transitions.

  @Example
    @code
    stepper(CW, HALF_STEP); 
 */
void step(int direction, int mode)
{
static const unsigned char step_code[8] = {8, 9, 1, 5, 4, 6, 2, 10}; 
static unsigned int step_ptr = 0;   // Initialize pointer only on power up

    step_ptr += (direction * mode);     // Compute step code pointer
    step_ptr &= sizeof(step_code)-1;	// force counter modulo 8
    
    if(step_code[step_ptr] & 0x01) LATBSET = A1IN1bit; else LATBCLR = A1IN1bit;           
    if(step_code[step_ptr] & 0x02) LATESET = A2IN1bit; else LATECLR = A2IN1bit;
    if(step_code[step_ptr] & 0x04) LATESET = B1IN1bit; else LATECLR = B1IN1bit;
    if(step_code[step_ptr] & 0x08) LATBSET = B2IN1bit; else LATBCLR = B2IN1bit;           
}

/*******************************************************************************
  Function:
    void __ISR(_TIMER_2_VECTOR, IPL3SOFT) Timer2Handler(void)

  Summary:
    Services a Timer 2 interrupt.

  Description:
    This interrupt service routine executes the required processing each Timer 2
    interrupt.  Two tasks are executed in this ISR: updates the seven-segment 
    LED display and generates a single stepper motor step. The seven-segment LED
    display display each of the four digits for one millisecond in a round-
    robin fashion. The stepper motor step is taken at the end of the step delay 
    period. Testing shows that this ISR takes between 2us and 4.25us.

  Precondition:
    IO has been initialized and global variables have been set.

  Parameters:
    None - ISR cannot have variables passed to it.

  Returns:
    None - ISR cannot return a value.

  Example:
    None. ISRs cannot be called directly. They can only be initiated after the 
    interrupt flag has been set either in software or hardware.
 
  Remarks:
    This routine uses and/or modifies the following global variables:
    led_flag, led_value, step_delay, step_mode, and step_dir.

    DB0 (RE0) is used to time the execution time of the ISR   

    The execution time of ISRs should be kept to a minimum to minimize latency
    of lower priority interrupts and foreground tasks.
  *****************************************************************************/
void __ISR(_TIMER_2_VECTOR, IPL3SOFT) Timer2Handler(void)
{
static int delay = 0;           // ms delay counter for stepper motor 

    if(--delay <= 0)                          // Test if step period over
    {
        delay = step_delay;                   // Reset the delay counter 
        --step_count;
        step(step_dir, step_mode);        // Move stepper motor 
        invLED3();                        // toggle step LED
    }
    INTClearFlag( INT_T2 );                   // Clear the interrupt flag
} /* End of Timer2Handler ISR */
/* *****************************************************************************
 End of File
 */
