/*
 * File:   audio.c
 * Author: rwall
 *
 * Created on May 2, 2016, 9:45 AM
 */
#include "Hardware.h"
#include <plib.h>
#include <math.h>
#include "audio.h"

static filter tone_filter;
static filter filt_r1;
static filter filt_r2;
static filter filt_r3;
static filter filt_r4;

static filter filt_r5;
static filter filt_r6;
static filter filt_r7;
static filter filt_r8;

int x[3] = {0};
int pwm_max;
int tones_active = FALSE;

static void pwm_analog(int ana);
static void initPWM(unsigned int update_rate);
static void select_filter(int tone);
static void calc_filters(int tone, filter *coeffs );


/* audio_init Function Description *******************************************
 * SYNTAX:          void audio_init(void);
 * KEYWORDS:        PWM, Audio, analog output
 * DESCRIPTION:     Initializes PWM for audio output
 * PARAMETERs:      None
 *
 * RETURN VALUE:    None
 *
 * NOTES:
 *
 * END DESCRIPTION **********************************************************/
void audio_init(void)
{
    calc_filters(FR1, &filt_r1);
    calc_filters(FR2, &filt_r2);
    calc_filters(FR3, &filt_r3);
    calc_filters(FR4, &filt_r4);

    calc_filters(FR5, &filt_r5);
    calc_filters(FR6, &filt_r6);
    calc_filters(FR7, &filt_r7);
    calc_filters(FR8, &filt_r8);

    initPWM( PWM_CYCLE_RATE );
}

/* initPWM Function Description **********************************************
 * SYNTAX:          void initPWM(int period);
 * KEYWORDS:        Pulse Width Modulation, PWM, Time 2
 * DESCRIPTION:     Initializes PWM output for OC1 using Timer 2 as time base.
 *                  Timer 2 is set for PBCLK frequency and to generate
 *                  interrupts at the PWM update frequency.
 * PARAMETER1:      int - Timer 2 PR
 * RETURN VALUE:    None
 *
 * NOTES:	See Section 13.1 page 185 of C32 Peripheral Library
 * 		Output for motor drive output control
 * END DESCRIPTION **********************************************************/
static void initPWM(unsigned int update_rate)
{
// reprogram RB14 for OC2
    DDPCONbits.JTAGEN = 0;  // Statement is required to use Pin RA0 as IO
//    ANSELBbits.ANSB14 = 0;  // Redundant for statement above
//    mOC1ClearIntFlag();		/* Clear any pending OC1 interrupts */
//    IFS0CLR = 0x00000040;   // CLear OC1 interrupt flag

/*  Setting up Timer 2 ******************************************************
 *     Parameter 1 - Configuration bits
 *     T2_ON        - Timer 2 set to on
 *     T2_PS_1_1    - Timer 2 pre-scale 1:1
 *     T2_SOURCE_INT - Timer 2 input source is PBCLK
 *     Parameter 2  - The PWM refresh period in ticks of the pre-scaled timer
 *                    input (PR2). For FBP (PBCLK) = 10E6 Hz and a 1 ms period,
 *                    PR2 = 10000. Resolution of PWM is 1 in 10000 (12 bit DAC).
**************************************************************************** */
//   pwm_max = (GetPeripheralClock() / update_rate);
    OpenTimer2(T2_ON | T2_PS_1_1 | T2_SOURCE_INT, (PWM_MAX  - 1));

//  This interrupts signals the end of (or beginning of next) PWM period
    mT2SetIntPriority( 2);	// Set Timer 2 group priority level 2
    mT2SetIntSubPriority( 1);	// Set Timer 2 subgroup priority level 1
    mT2IntEnable( 1);		// Enable T2 interrupts

    PORTSetPinsDigitalOut(IOPORT_B, BIT_8);  // May not be needed
    RPB7R = 0x0B;           //  Set RB7 for OC5
    OC5CONbits.ON = 0;      // Reset OC5 control
    OC5R = 3000;
    OC5RS = 3000;
    OC5CON = 0x0006;        // Set OC5 control for PWM

    IFS0bits.OC5IF = 0;
    IPC5bits.OC5IP = 7;     // Priority 7
    IPC5bits.OC5IS = 3;     // Sub Priority 3
    IEC0bits.OC5IE = 1;     // Enable interrupts
    OC5CON |= 0x8000;       // Turn on OC5

    PORTSetPinsDigitalOut(IOPORT_B, A_OUTbit);  // May not be needed    
    RPB14R = 0x0C;          // Set RB14 for OC1 - PIC3xx
    OC1CONbits.ON = 0;      // Reset OC5 control
    OC1R = 6000;
    OC1RS = 6000;
    OC1CON = 0x0006;        // Set OC5 control for PWM

    IFS0bits.OC1IF = 0;
    IPC1bits.OC1IP = 3;     // Priority 7
    IPC1bits.OC1IS = 0;     // Sub Priority 3
    IEC0bits.OC1IE = 1;     // Enable interrupts
    OC1CON |= 0x8000;       // Turn on OC5
    
    SetDCOC1PWM(PWM_ZERO_OFFSET);   // Start PWM at 0 VDC
} /* End of initPWM */

/* pwm_analog Function Description *****************************************
 * SYNTAX:          void pwm_analog(int ratio);
 * KEYWORDS:        PWM, analog output
 * DESCRIPTION:     Sets the PWM ratio for controlling the PWM output by
 *                  adjusting the portion of PR2 that the output, OC3,
 *                  is high.
 * PARAMETER1:      analog output
 * RETURN VALUE:    None
 * NOTES:           None
 * END DESCRIPTION **********************************************************/
static void pwm_analog(int ana)
{
    SetDCOC1PWM(ana);		/* Set PWM ratio */
    SetDCOC5PWM(ana);		/* Set PWM ratio */
}

/* gen_tones FUNCTION DESCRIPTION *****************************************
 * SYNTAX:          static void gen_tones(void);
 * KEYWORDS:        DTMF, IIR filter, digital oscillator
 * DESCRIPTION:     Generates a set of DTMF frequencies
 *                  Set the sample flag to TRUE.
 * RETURN VALUE:    None
 * Notes:           This works only if the step function requires less than
 *                  0.125 ms (8.0 KHz)
 * END DESCRIPTION ***********************************************************/
static void gen_tones(void)
{
//static int pwmmax = PWM_MAX; 
int pwm;

/* Oscillator IIR filter */
    x[2] = x[1];        /* Shift row channel outputs and compute new output */
    x[1] = x[0];
    x[0] = (tone_filter.a1 * x[1] - tone_filter.a2 * x[2]);
    
    if(x[0] < 0)
    {
        x[0] = x[0] - Q14/2;
    }
    else
    {
        x[0] = x[0] + Q14/2;        
    }
    x[0] = x[0] / Q14;

/* Shift column channel outputs and compute new output */
    pwm = (x[0] * PWM_MAX / Q14 ) + (PWM_MAX / 2);

    if(pwm < 0)
    {
        pwm = 0;
    }
    if(pwm > (PWM_MAX - 1) )
    {
        pwm = PWM_MAX - 1 ;
    }
    pwm_analog(pwm);       /* Send scaled ch1 output to PWM */
}


/* Timer2Handler FUNCTION DESCRIPTION *****************************************
 * SYNTAX:          Timer2Handler(void);
 * KEYWORDS:        Millisecond, hardware delay, interrupt, Timer2, ISR
 * DESCRIPTION:     Toggles LEDB each interrupt.
 *                  Set the sample flag to TRUE.
 * RETURN VALUE:    None
 * Notes:           This works only if the step function requires less than
 *                  0.125 ms
 * END DESCRIPTION ***********************************************************/
void __ISR(_TIMER_2_VECTOR, IPL2SOFT) Timer2Handler(void)
{
    if(tones_active == TRUE)    /* Call IIR filter only if turned on */
        AIN1(1);
        gen_tones();
        AIN1(0);
    INTClearFlag( INT_T2 );     /* Clear the interrupt flag */
} /* End of Timer2Handler ISR */

void __ISR(_OUTPUT_COMPARE_5_VECTOR, IPL7SOFT) OC5ISRHandler(void)
{
    INTClearFlag( INT_OC5 );     // Clear the interrupt flag
} // End of OC5ISRHandler

void __ISR(_OUTPUT_COMPARE_1_VECTOR, IPL7SOFT) OC1ISRHandler(void)
{
    INTClearFlag( INT_OC1 );     // Clear the interrupt flag
} // End of OC1ISRHandler


/*
#pragma interrupt OC1IntHandler ipl4 vector 36
void OC1IntHandler(void)
{
// insert user code here
    IFS0CLR = 0x00000040; // Clear the interrupt flag
}
*/

/* ------------------- Set Oscillator frequency ------------------------ */
void start_tones(int tone)
{
    tones_active = FALSE;           /* Disable Timer ISR oscillator */
    select_filter(tone);        /* Set filter coefficients */

    x[0] = tone_filter.a0;           /* Initialize Row oscillator filter */
    x[1] = 0;
    x[2] = 0;

    tones_active = TRUE;            /* Ensable Timer ISR oscillator */
}

/* select_filter FUNCTION DESCRIPTION *****************************************
 * SYNTAX:          static void select_filter(int row, int col);
 * KEYWORDS:        DTMF, IIR filtter, digital oscillator
 * DESCRIPTION:     Generates a set of DTMF frequencies
 * Parameter 1:     Row number
 * Parameter 2:     Column number
 * RETURN VALUE:    None
 * Notes:           This works only if the step function requires less than
 *                  0.125 ms (8KHz)
 * END DESCRIPTION ***********************************************************/
/* Select oscillator constants according to row and column number */
static void select_filter(int tone)
{
    switch(tone)
    {
        case 1:     // 500 Hz             
            tone_filter.a0 = filt_r1.a0;
            tone_filter.a1 = filt_r1.a1;
            tone_filter.a2 = filt_r1.a2;
            break;
        case 2:     // 1500 Hz
            tone_filter.a0 = filt_r2.a0;
            tone_filter.a1 = filt_r2.a1;
            tone_filter.a2 = filt_r2.a2;
            break;
        case 3:     // 2500 Hz             
            tone_filter.a0 = filt_r3.a0;
            tone_filter.a1 = filt_r3.a1;
            tone_filter.a2 = filt_r3.a2;
            break;
        case 4:     // 3500 Hz             
            tone_filter.a0 = filt_r4.a0;
            tone_filter.a1 = filt_r4.a1;
            tone_filter.a2 = filt_r4.a2;
            break;
        case 5:     // 4500 Hz             
            tone_filter.a0 = filt_r5.a0;
            tone_filter.a1 = filt_r5.a1;
            tone_filter.a2 = filt_r5.a2;
            break;
        case 6:     // 5500 Hz             
            tone_filter.a0 = filt_r6.a0;
            tone_filter.a1 = filt_r6.a1;
            tone_filter.a2 = filt_r6.a2;
            break;
        case 7:     // 6500 Hz             
            tone_filter.a0 = filt_r7.a0;
            tone_filter.a1 = filt_r7.a1;
            tone_filter.a2 = filt_r7.a2;
            break;
        case 8:     // 7500 Hz             
            tone_filter.a0 = filt_r8.a0;
            tone_filter.a1 = filt_r8.a1;
            tone_filter.a2 = filt_r8.a2;
            break;
        default:    // No oscillations
            tone_filter.a0 = 0;
            tone_filter.a1 = 0;
            tone_filter.a2 = 0;
            break;
    }
}

static void calc_filters(int tone, filter *coeffs )
{
// H(z) = z(sin(wo*T)) / (z^2 - 2*z*cos(wo*T) + 1)

float w; 
    w = (((float) (tone)) * PIx2)/PWM_CYCLE_RATE;
    coeffs->a0 = sin(w) * Q14/2;    // Limit initial impulse to 1/2
    coeffs->a1 = 2*cos(w) * Q14;
    coeffs->a2 = Q14; 
}