/* ************************************************************************** */
/** Descriptive File Name: FFT.c - FFT algorithm uses the Mips DSP library

  @ Author
    Richard Wall
 
  @ Date
    Created;    April 30, 2016, 9:58 AM
    Modified:   February 19, 2017        
    Verified:   May 19, 2017

  @Company
    Digilent Inc

  @Development Environment
    MPLAB X IDE x3.61 - http://www.microchip.com/mplab/mplab-x-ide 
	XC32 1.43 - http://www.microchip.com/mplab/compilers
	PLIB 3/7/20162 - http://www.microchip.com/SWLibraryWeb/product.aspx?product=PIC32%20Peripheral%20Library

  @File Name
    fft.c

  @Summary
    Fast Fourier Transform

  @Description
    Computes the Fast Fourier Transform of a block of length specified by
    FFTLEN.
 
  @Remarks
    Uses XC32 peripheral library. See 
 http://ww1.microchip.com/downloads/en/DeviceDoc/32bitPeripheralLibraryGuide.pdf
 http://www.microchip.com/SWLibraryWeb/product.aspx?product=PIC32%20Peripheral%20Library
 
 */
/* ************************************************************************** */

// Basys MX3
#include "hardware.h"
#include <plib.h>
#include <math.h>
#include <stdint.h>
#include <stdio.h>

// DSP 
#include "fft.h"
#include <dsplib_dsp.h>
#include <fftc.h>
#include <limits.h>

// Global variables used in main.c
int16c InData[FFTLEN];      // Time domain input signal
int16c OutF[FFTLEN];        // Frequency domain FFT result
int16c OutT[FFTLEN];        // and inverse FFT back to the time domain
int16  BlackWindow[FFTLEN]; // Our windowing function

/*************************************************************************
MIPS FFT Algorithm. 
		int16c *FOut - 2 X N array of complex frequensy results
		int16c *Tin  - 2 X N array of complex time domain data samples
Notes: 
    FFT algorithm uses the mips DSP library

    Code based on: 
http://www.microchip.com/forums/tm.aspx?tree=true&high=&m=437722&mpage=1#437722 
    that uses the mips_fft16 or the mips_fft32 library functions needs only 26 
    microseconds.
 
    FFT sizes must be power of 2. 

	Twiddle factors, fftc, are provided by the declaration
	Parameters:	#define fftc fft16c32 // from fftc.h, for N = 32

	Scratch pad buffer need not be preserved between itterations.
   
    #include fftc.h?        // pre-computed coefficients
    #define fftc fft16c32   // from fftc.h, for N = 32

    int log2N = 5; // log2(32) = 5
    int N = 1 << log2N; // N = 2^5 = 32
    int16c din[N];
    int16c dout[N];
    int16c scratch[N];
	
 *  A 32 point FFT requires 62 uS.
 * 
 *************************************************************************/

void FFT(int16c *FOut, int16c *TIn)
{
int16c Scratch[FFTLEN];
    mips_fft16( FOut, TIn, (int16c *)fftc, Scratch, LOG2FFTLEN);
}

/* FFTAmplitude FUNCTION DESCRIPTION *************************************
SYNTAX:         uint16_t FFTAmplitude(int16_t re, int16_t im);
KEYWORDS:       FFT, complex, polar, absolute value, magnitude, degrees
DESCRIPTION:	Computes the absolute value of a complex value
PARAMETER1:     Real part of comples value
PARAMETER1:     Imaginary part of comples value
RETURN VALUE:   int16_t amplitude
NOTES:          A = SQRT(Re^2 + Im^2)
                http://www.codecodex.com/wiki/Calculate_an_integer_square_root#C 
END DESCRIPTION **********************************************************/
#ifndef NO_FLOAT
uint16_t FFTAmplitude(int16_t re, int16_t im)
{
uint32_t _bit = ((uint32_t)1) << 30;
uint32_t num;
uint32_t res = 0;

    num = ((uint32_t)re)*re + ((uint32_t)im)*im;

// 32 bit integer square root calculation.
    while (_bit > num)
        _bit >>= 2;

    while (_bit != 0) 
    {
        if (num >= res + _bit) 
        {
            num -= res + _bit;
            res = (res >> 1) + _bit;
        }
        else
            res >>= 1;
        _bit >>= 2;
    }
    return ((uint16_t)res);
}
#endif

/* FFTPhase FUNCTION DESCRIPTION *************************************
SYNTAX:         int16_t FFTPhase(int16_t re, int16_t im);
KEYWORDS:       FFT, complex, polar, phase, degrees
DESCRIPTION:	Computes the phase in degrees of a complex value
PARAMETER1:     Real part of comples value
PARAMETER1:     Imaginary part of comples value
RETURN VALUE:   int16_t phase in degrees
NOTES:          
 *              D = (180/Pi)*ARCTAN(Im/Re)
 * Second order approximation:
 *  atan_approx(x) = (Pi/2)*(b*x + x*x)/(1 + 2*b*x + x*x)
 *  where b = 0.596227, with a maximum approximation error of 0.1620
 * Third order:
 *  atan_approx(x) = (Pi/2)*(c*x + x*x + x*x*x)/(1 + (c+1)*x + (c+1)x*x + x*x*x)
 *  where c = (1 + sqrt(17))/8 and the maximum approximation error is 0.00811
 *
END DESCRIPTION **********************************************************/
#ifndef NO_FLOAT
int16_t FFTPhase(int16_t re, int16_t im)
{
/* 
typedef union
{
    uint32_t w;
    struct
    {
        uint8_t lo;
        uint8_t hi;
        uint8_t up;
        uint8_t reserved;
    };
} DATA;
*/
// constants in 16.16 bit fixed point format
#define POINT_POS  8
#define ONE        (((uint32_t)1)<<POINT_POS)
#define HALF       ((uint32_t)(ONE*0.5))
#define B          ((uint32_t)(ONE*0.596227))

int32_t bx;
int32_t xx;
int32_t result;
int32_t x;
uint16_t sign = 0;
uint16_t quadrant = 0;

   if(im == 0)
   {
       return 0;
   }

   // im/re
   if(re == 0)
   {
       if(im < 0)
       {
           return ((int16_t)-90);
       }else{
           return ((int16_t)90);
       }
   }
   x = im;
   x <<= POINT_POS;
   x /= re;

   // arctan(-x) = -arctan(x)
   if(x < 0)
   {
       sign = 1;
       x = -x;
   }

   if(re < 0)
   {
       quadrant = 1;
   }

   // b*x
   bx = B;
   bx *= x;
   // shift after multiplication
   bx >>= POINT_POS;

   // x^2
   xx = x;
   xx *= x;
   // shift after multiplication
   xx >>= POINT_POS;

   // bx + x^2
   result = bx;
   result += xx;

   // (bx + x^2)/(1+2bx+x^2)
   // shift before division
   result <<= POINT_POS;
   result /= (ONE + (bx<<1) + xx);

   result *= 90;

   if(result&HALF)
   {
       result += ONE;
   }

   result >>= POINT_POS;

   if(quadrant)
   {
       // add 90 degree
       result += 90;
   }

   if(sign)
   {
       return ((int16_t)-result);
   }

   return ((int16_t)result);
}
#endif

/* WindowBlackmanHarrisInit FUNCTION DESCRIPTION *****************************
SYNTAX:         void WindowBlackmanHarrisInit(int16 *pas16, int nLen);
KEYWORDS:       data, window, 
DESCRIPTION:	Computes the coefficients for a Blackman-Harris window. 
PARAMETER1:     Pointer to array int16 coefficients
PARAMETER2:     Integer window size
RETURN VALUE:   none
NOTES:          Refer to:
  https://www.dsprelated.com/freebooks/sasp/Blackman_Harris_Window_Family.html 
 
END DESCRIPTION **********************************************************/
#ifndef NO_FLOAT
void WindowBlackmanHarrisInit(int16 *pas16, int nLen)
{
int i;
    for (i=0;i<nLen;i++)
    {
        pas16[i]=(int16)(SHRT_MAX*(0.35875 - 0.48829*cos(2*PI*i/(nLen-1)) +\
                  0.14128*cos(4*PI*i/(nLen-1)) - 0.01168*cos(6*PI*i/(nLen-1))));
    } 
}
#endif

/* WindowBlackmanHarrisInit FUNCTION DESCRIPTION *****************************
SYNTAX:         void WindowBlackmanHarrisInit(int16 *pas16, int nLen);
KEYWORDS:       data, window, 
DESCRIPTION:	Computes the array of data samples processed by a 
                Blackman-Harris window. 
PARAMETER1:     Pointer to array int16c complex data elements
PARAMETER2:     Integer window size
RETURN VALUE:   none
NOTES:          Refer to:
  https://www.dsprelated.com/freebooks/sasp/Blackman_Harris_Window_Family.html 
 
END DESCRIPTION **********************************************************/
#ifndef NO_FLOAT
void WindowBlackmanHarris(int16c *InData, int nLen)
{
int i;
    for (i=0; i<nLen; i++)
    {
        InData[i].re=(int16)((((int64)InData[i].re) * BlackWindow[i])>>15);
        InData[i].im=(int16)((((int64)InData[i].im) * BlackWindow[i])>>15);
    }  
}
#endif

/* FFTLog FUNCTION DESCRIPTION *************************************
SYNTAX:         int16_t FFTLog(int16_t amplitude);
KEYWORDS:       FFT, complex, polar, log base 10
DESCRIPTION:	Computes the Integer loge base 10 of a real value
PARAMETER1:     Real value
RETURN VALUE:   int16_t log base 10 value
NOTES:          
END DESCRIPTION **********************************************************/
int16_t FFTLog(int16_t amplitude)
{
int16_t power;
/*
int v;
    v = ((int) amplitude) << 2;
    power = (v >= 16000u) ? 15 : (v >= 8000u) ? 14 : 
            (v >= 4000u) ? 13 : (v >= 2000u) ? 12 : 
            (v >= 1000u) ? 11 : (v >= 500u) ? 10 :
            (v >= 256u) ? 9 : (v >= 120u) ? 8 : (v >= 60u) ? 7u : 
            (v >= 30u) ? 6 : ((v >= 15u) ? 5 : (v >= 8u) ? 4 :
            (v >= 4u) ? 3 : (v >= 2u) ? 2 :(v >= 1u) ? 1 : 0u); 
*/
    power = amplitude / 4;
    if(power > 15)
    {
        power = 15;
    }
    return power;
}
// End of fft.c
