/* ************************************************************************** */
/** Descriptive File Name: RC.c - Remote servo controller demonstration program

  @ Author  
     Richard Wall
  
  @ Date: 
     Created:    November 24, 2012
     Revised:    March 3, 2017 - ported to PIC32MX370 for the Basys MX3 
     Verified:	 May 21, 2017
	 
  @Company
    Digilent Inc

  @File Name
    RC.c

  @Development Environment
    MPLAB X IDE x3.61 - http://www.microchip.com/mplab/mplab-x-ide 
	XC32 1.43 - http://www.microchip.com/mplab/compilers
	PLIB 3/7/20162 - http://www.microchip.com/SWLibraryWeb/product.aspx?product=PIC32%20Peripheral%20Library

  @Summary
    Test program for the 2 channel Servo Motor using the Basys MX3.

  @Description
    See RC.txt under the Reference folder

  @ Remarks:
    An external 5VDC is required for servo power
    
    This implementation uses a timer counter to generate the servo proportional
    position signal.
**************************************************************************** */

// System included files
#include "config_bits.h"
#include "hardware.h"

// XC32 libraries
#include <plib.h>
#include <string.h>
#include <stdio.h>
#include <stdint.h>

// Included files for platform 
#include "LCDlib.h"
#include "comm_lib.h"
#include "swDelay.h"

// Included files for application 
#include "RC.h"

int rc[NRC];                    // Array of RC Channel position Range = 0-100
char BLANK_LINE[] = "                    ";
int switch_flag = TRUE;                 // Force LCD update after reset

/* ----------------------------------------------------------- */
int main(void)
{
char lcdstr[40];                        // Local LCD display buffers
char uartstr[80];                       // Local UART input buffer
int ch, pos;                            // Used for UART entry

	Hardware_Setup();                   // Initialize common Basys MX3 hardware
	initLCD();                          // Initialize LCD
	uart4_init(38400, NO_PARITY);       // Initialize UART4
	initRC();                           // Initialize RC channel output pins
	clrLCD();                           // Clear hello messages
	putsLCD("RC Demo");                 // Hello messages
	printf("\n\rDr. Wall's RC Demo - May 21, 2017\n\r");
	printf("Enter servo channel and position\n\r");
	msDelay(2000);                      // Wait two seconds
	check_switches();                   // Initialize RC times
	initTimer2();                       // Start timer interrupt
	clrLCD();                           // Clear hello messages
	while(1)
	{
		if(switch_flag)                 // Update HMIs if the button state changes
		{	
			switch_flag = FALSE;;
			sprintf(lcdstr,"%3d %3d",rc[0],rc[1]);
			gotoLCD(0);                 // Blank out LCD line 1
			putsLCD(BLANK_LINE);
			gotoLCD(0);
			putsLCD(lcdstr);            // Post channel settings to LCD line 1
			printf("%s\n\r", lcdstr);	// Send channel setting to UART
		}

		if(getstrU4( uartstr, strlen(uartstr) ) > 0) // Test for serial string ready
		{
			gotoLCD(16);                // Blank out LCD line 1
			putsLCD(BLANK_LINE);
			gotoLCD(16);                // Blank out LCD line 1
			putsLCD(uartstr);           // Copy string to LCD line 2
			sscanf(uartstr,"%d %d",&ch, &pos); // Decode entry
			if(ch >= 0 && ch < NRC)     // Check RC channel number range
			{
				if(pos >=0 && pos <= 100)	// Check RC position range
				{
					rc[ch] = pos*RC_SPAN/100; // Update RC channel position
					switch_flag = TRUE;     // Force update of LCD Line 1
				}
			}		
		}
	}
	return EXIT_FAILURE;		// The program should never get to this LOC 
}

/* START FUNCTION DESCRIPTION ********************************************
initRC
SYNTAX:  		void initRC{void);
KEYWORDS:		initialize, RC, outputs
DESCRIPTION:	Initialize the RC outputs
RETURN VALUE:   none
NOTES:			See RC.h for pin definitions
END DESCRIPTION **********************************************************/
void initRC(void)
{
	cfgRC1();		// Set RC pins for output
	cfgRC2();		
    RC_1(0);        // Reset all RC pins to zero
    RC_2(0);
}

/* START FUNCTION DESCRIPTION ********************************************
initTimer2
SYNTAX:  		void initTimer2(void);
KEYWORDS:		initialization system timer 2
DESCRIPTION:	Sets timer 2 for an 10 us interrupt
RETURN VALUE:   none
NOTES:			None
END DESCRIPTION **********************************************************/

void initTimer2(void)
{
#define TMR2_TICK	100

    OpenTimer2(T2_ON | T2_SOURCE_INT | T2_PS_1_1, TMR2_TICK-1);
// Set Timer 2 interrupt with a priority of 2
    ConfigIntTimer2(T2_INT_ON | T2_INT_PRIOR_2);
}

/* START FUNCTION DESCRIPTION ********************************************
Timer2Handler
SYNTAX:			Timer2Handler(void);
KEYWORDS:		hardware delay, interrupt, Timer 2
DESCRIPTION:   	Processes the code associated with a Timer 2 interrupt.
				It is responsible for generating the signals for each 
				RC channel in two phases: The three state FSM control the
				on period and the off period.
PARAMETERS:		None are allowed for an ISR
RETURN VALUE:  	None are allowed for an ISR
Notes:			This function cannot be called from any other function.
END DESCRIPTION **********************************************************/
void __ISR(_TIMER_2_VECTOR, IPL2SOFT) Timer2Handler(void)
{
static int channel = 0;			// Channel index 
static int rc_state = 0;		// Initial signal state
static int rc1, rc2;			// Signal period timers

    mT2ClearIntFlag();			// clear the interrupt flag
	
	switch(rc_state)
	{
		case 0:		// Initial high period - rc1 and rc2 are set here
			set_RC(channel, TRUE);		// Turn channel on
			rc1 = RC_MIN + rc[channel];	// Compute on time
			if(rc1 > RC_MAX)			// Check limits
				rc1 = RC_MAX;
			rc2 = RC_MAX - rc1;			// Time to end of cycle
			if(rc2 < 0)					// Check limits
				rc2 = 0;
			rc_state++;					// Set for next state
			break;							
		case 1:
			if(--rc1 <= 0)				// Count down cycle ON time
			{
				set_RC(channel, FALSE);	// Turn channel off
				rc_state++;				// Set for next state
			}
			break;
		case 2:
			if(--rc2 <= 0)				// Count down Cycle off time
			{
				rc_state = 0;			// Reset to initial state 
				channel = (channel +1) % NRC;  // Next channel
				if(channel == 0)		// Update channel times only once a cycle
					switch_flag = check_switches();	// Use set inputs	
			}
			break;
	}
}

/* START FUNCTION DESCRIPTION ********************************************
SYNTAX:			void set_RC(int ch, int ctrl);
KEYWORDS:		RC, control, output
DESCRIPTION:   	This function sets the RC channel specified by "ch" to the 
				the condition specified by "ctrl". Using macros to define the
				IO pin allows the developer to specify an arbitrary processor
				pin to each RC channel.
PARAMETER1:		channel number
PARAMETER2:		control - OFF - 0,  ON - 1  
RETURN VALUE:  	None
Notes:			Switch-case can be expanded to accommodate  additional 
				channels.  Most systems are limited to 10 channels
END DESCRIPTION **********************************************************/
void set_RC(int ch, int ctrl)
{
	switch(ch)
	{
		case 0:
			RC_1(ctrl);
			break;
		case 1:
			RC_2(ctrl);
			break;
// add additional channels here.
		default:
			break;
	}
}

/* START FUNCTION DESCRIPTION ********************************************
SYNTAX:			int check_switches(void);
KEYWORDS:		switches, decode
DESCRIPTION:   	This function polls the three bush buttons on the 
				Basys MX3 board and sets the RC positions
				to a common value. The RC channel positions are not changed
				unless there has been a change of button state.   
PARAMETERS:		None
RETURN VALUE:  	1 = new switch condition, 0 = no new switch condition
Notes:			The switches on are encoded using a 1 hot code.  
				SW0 = 0/1, SW1 = 0/2, and SW2 = 0/4;
END DESCRIPTION **********************************************************/
int check_switches(void)
{
static int s2 = 0xff;	// force setting RC channels when initialized
int s1; 		// s1 does not need to be static - the newest status
int setting;	// RC servo position setting
int ch;			// RC channel index
	
	s1 = ((int) SW0()) | ((int) SW1()<<1) | ((int) SW2()<<2);	

	if(s1 != s2)  // Is this a new switch setting?
	{
		switch(s1)	// Set all RC units to the same setting
		{
			case 0: 
				setting = RC_SPAN/2;
				break;
			case 1:				
				setting = 0;
				break;
			case 2:				
				setting = RC_SPAN/4;
				break;
			case 4:				
				setting = (3*RC_SPAN)/4;
				break;
			default:
				setting = RC_SPAN;
		}

		for(ch = 0; ch<NRC; ch++)	// Set all RC channels to the same setting
		{
			rc[ch] = setting;
		}
		s2 = s1;                    // Remember this switch setting
		return TRUE;                // switch CoS
	}
	else
		return FALSE;               // No switch CoS
}
